@@ -4,6 +4,7 @@ import os
 import warnings
 from collections import Counter
 from fnmatch import fnmatch
+from importlib import metadata
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
@@ -25,33 +26,31 @@ from psygnal import Signal, SignalGroup
 
 from ._command_registry import CommandRegistry
 from .manifest import PluginManifest
-from .manifest.writers import LayerType, WriterContribution
+from .manifest._npe1_adapter import NPE1Adapter
+from .manifest.contributions import LayerType, WriterContribution
 from .types import PathLike, PythonName, _ensure_str_or_seq_str
 
 if TYPE_CHECKING:
-    from .manifest.commands import CommandContribution
-    from .manifest.menus import MenuItem
-    from .manifest.readers import ReaderContribution
-    from .manifest.sample_data import SampleDataContribution
-    from .manifest.submenu import SubmenuContribution
-    from .manifest.themes import ThemeContribution
-    from .manifest.widgets import WidgetContribution
+    from .manifest.contributions import (
+        CommandContribution,
+        MenuItem,
+        ReaderContribution,
+        SampleDataContribution,
+        SubmenuContribution,
+        ThemeContribution,
+        WidgetContribution,
+    )
 
 __all__ = ["PluginContext", "PluginManager"]
 PluginName = str  # this is `PluginManifest.name`
 
-try:
-    from importlib import metadata
-except ImportError:
-    import importlib_metadata as metadata  # type: ignore
-
 
 class _ContributionsIndex:
     def __init__(self) -> None:
         self._indexed: Set[str] = set()
         self._commands: Dict[str, Tuple[CommandContribution, PluginName]] = {}
-        self._readers: List[Tuple[str, ReaderContribution]] = list()
-        self._writers: List[Tuple[LayerType, int, int, WriterContribution]] = list()
+        self._readers: List[Tuple[str, ReaderContribution]] = []
+        self._writers: List[Tuple[LayerType, int, int, WriterContribution]] = []
 
         # DEPRECATED: only here for napari <= 0.4.15 compat.
         self._samples: DefaultDict[str, List[SampleDataContribution]] = DefaultDict(
@@ -224,6 +223,7 @@ class PluginManager:
         self._contrib = _ContributionsIndex()
         self._manifests: Dict[PluginName, PluginManifest] = {}
         self.events = PluginManagerEvents(self)
+        self._npe1_adapters: List[NPE1Adapter] = []
 
         # up to napari 0.4.15, discovery happened in the init here
         # so if we're running on an older version of napari, we need to discover
@@ -254,7 +254,9 @@ class PluginManager:
 
     # Discovery, activation, enablement
 
-    def discover(self, paths: Sequence[str] = (), clear=False) -> None:
+    def discover(
+        self, paths: Sequence[str] = (), clear=False, include_npe1=False
+    ) -> None:
         """Discover and index plugin manifests in the environment.
 
         Parameters
@@ -266,6 +268,9 @@ class PluginManager:
             Clear and re-index the environment.  If `False` (the default),
             calling discover again will only register and index newly
             discovered plugins. (Existing manifests will not be re-indexed)
+        include_npe1 : bool
+            Whether to detect npe1 plugins as npe1_adapters during discovery.
+            By default `False`.
         """
         if clear:
             self._contrib = _ContributionsIndex()
@@ -273,9 +278,19 @@ class PluginManager:
 
         with self.events.plugins_registered.paused(lambda a, b: (a[0] | b[0],)):
             for result in PluginManifest.discover(paths=paths):
-                if result.manifest and result.manifest.name not in self._manifests:
+                if (
+                    result.manifest
+                    and result.manifest.name not in self._manifests
+                    and (include_npe1 or not isinstance(result.manifest, NPE1Adapter))
+                ):
                     self.register(result.manifest, warn_disabled=False)
 
+    def index_npe1_adapters(self):
+        with warnings.catch_warnings():
+            warnings.showwarning = lambda e, *_: print(str(e).split(" Please add")[0])
+            while self._npe1_adapters:
+                self._contrib.index_contributions(self._npe1_adapters.pop())
+
     def register(self, manifest: PluginManifest, warn_disabled=True) -> None:
         """Register a plugin manifest"""
         if manifest.name in self._manifests:
@@ -288,6 +303,8 @@ class PluginManager:
                     f"Disabled plugin {manifest.name!r} was registered, but will not "
                     "be indexed. Use `warn_disabled=False` to suppress this message."
                 )
+        elif isinstance(manifest, NPE1Adapter):
+            self._npe1_adapters.append(manifest)
         else:
             self._contrib.index_contributions(manifest)
         self.events.plugins_registered.emit({manifest})
