@@ -1,5 +1,7 @@
 from pathlib import Path
 
+import pytest
+
 from npe2 import PluginManifest
 
 DOCS_DIR = Path(__file__).parent.parent / "_docs"
@@ -10,6 +12,7 @@ def test_example_manifest():
     assert PluginManifest.from_file(example)
 
 
+@pytest.mark.github_main_only
 def test_render_docs(tmp_path, monkeypatch):
     from _docs.render import main
 
